C$Procedure  VRTCOM  ( given a vertex find the plates )
 
      SUBROUTINE VRTCOM ( NP, V1, V2, V3, VCPTR, NVC, VCLIST )
 
C$ Abstract
C
C     Generate an array containing the plates that belong
C     to a specific vertex.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     PLATE VERTEX
C
C$ Declarations
 
      IMPLICIT NONE
 
      INTEGER               NP
      INTEGER               NVC
      INTEGER               V1      ( NP )
      INTEGER               V2      ( NP )
      INTEGER               V3      ( NP )
      INTEGER               VCPTR   ( * )
      INTEGER               VCLIST  ( * )
 
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NP         I   Number of plates.
C     V1         I   Array of first vertex ID's for each plate.
C     V2         I   Array of second vertex ID's for each plate.
C     V3         I   Array of third vertex ID's for each plate.
C     VCPTR      O   Array of pointers, by vertex, into plate data list.
C     NVC        O   Length of the plate data list.
C     VCLIST     O   Plate data list; first element is the number of
C                    plate ID's to follow.
C
C$ Detailed_Input
C
C     NP         Number of plates.
C
C     V1         Array of first vertex ID's for each plate.
C
C     V2         Array of second vertex ID's for each plate.
C
C     V3         Array of third vertex ID's for each plate.
C
C$ Detailed_Output
C
C     VCPTR      Array of pointers, by vertex, into plate data list.
C
C     NVC        Length of the plate data list.
C
C     VCLIST     Plate data list; first element is the number of
C                plate ID's to follow.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C     1)
C
C$ Particulars
C
C     Later.
C
C$ Examples
C
C     Later.
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     J.A. Bytof      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 04-FEB-1999 (JAB)
C
C
C-&
 
C$ Index_Entries
C
C     given a vertex find the plates it belongs to
C
C-&
 
      INCLUDE              'pltmax.inc'
 
C
C     Local variables.
C
 
      LOGICAL               RETURN
 
      INTEGER               I
      INTEGER               NCELL
      INTEGER               PNTRS   ( MAXNPV )
      INTEGER               BARR    ( MAXNPV )
      INTEGER               CELLS   ( 3, MAXNPV )
 
C
C     Standard SPICELIB error handling.
C
 
      IF ( RETURN () ) THEN
         RETURN
      END IF
 
      CALL CHKIN  ( 'VRTCOM' )
 
C
C     Initialize pointer and cell structure.
C
      CALL INILNK ( MAXNPV, MAXNPV, NCELL, PNTRS, CELLS )
 
C
C     Add each plate/vertex combination to the linked list.
C
      DO I = 1, NP
         CALL ADDLNK ( V1(I), I, PNTRS, NCELL, CELLS )
         CALL ADDLNK ( V2(I), I, PNTRS, NCELL, CELLS )
         CALL ADDLNK ( V3(I), I, PNTRS, NCELL, CELLS )
      END DO
 
C
C     Generate two lists:
C
C     1. A list of pointers for each vertex into the data list.
C
C     2. The data list, containing for each vertex the number
C        of plates that contains that vertex, followed by the
C        list of plate ID's.
C
 
 
      CALL UNTNGL ( NCELL, PNTRS, MAXNPV, BARR, CELLS, VCPTR,
     .              NVC, VCLIST )
 
 
C
C     Standard SPICE error handling.
C
 
      CALL CHKOUT ( 'VRTCOM' )
 
 
      RETURN
      END
