      PROGRAM TPNTOCC
 
C
C     Test routine for PNTOCC.
C
 
      IMPLICIT              NONE
 
      INCLUDE              'pltmax.inc'
 
      INTEGER               FVROOM
      PARAMETER           ( FVROOM = 20 )
 
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
 
      DOUBLE PRECISION      NSIDES
      PARAMETER           ( NSIDES = 16.0D0 )
 
      LOGICAL               AWAY
 
      INTEGER               FUNIT
      INTEGER               I
      INTEGER               J
      INTEGER               IAW
      INTEGER               IFOV
      INTEGER               IVIS
      INTEGER               ILIT
      INTEGER               NB
      INTEGER               NV
      INTEGER               NP
      INTEGER               OBSID
      INTEGER               INSTID
      INTEGER               NDIV
      INTEGER               ITMP
      INTEGER               TRIDIV
      INTEGER               PMASK
      INTEGER               FOVSIZ
 
      DOUBLE PRECISION      VSEP
      DOUBLE PRECISION      TWOPI
      DOUBLE PRECISION      ANGLE
      DOUBLE PRECISION      FOVAMT
      DOUBLE PRECISION      VISAMT
      DOUBLE PRECISION      LITAMT
      DOUBLE PRECISION      BORDIR ( 3 )
      DOUBLE PRECISION      BDSITE ( 3 )
      DOUBLE PRECISION      BSIGHT ( 3 )
      DOUBLE PRECISION      NRMSUN
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      MAXELE 
      DOUBLE PRECISION      FOVSAV ( MAXPLT )
      DOUBLE PRECISION      VISSAV ( MAXPLT )
      DOUBLE PRECISION      LITSAV ( MAXPLT )
      DOUBLE PRECISION      OBS    ( 3 )
      DOUBLE PRECISION      LOOK   ( 3 )
      DOUBLE PRECISION      SUN    ( 3 )
      DOUBLE PRECISION      NRML   ( 3 )
      DOUBLE PRECISION      FOV    ( 3, FVROOM )
      DOUBLE PRECISION      FOVNRM ( 3, FVROOM )
      DOUBLE PRECISION      XROT   ( 3, 3 )
      DOUBLE PRECISION      EDGE   ( 3, FVROOM )
      DOUBLE PRECISION      SIDE   ( 3, FVROOM )
 
      CHARACTER*(2)         ANA
      CHARACTER*(2)         ARTCLE
      CHARACTER*(1)         FYN
      CHARACTER*(80)        IKERN
      CHARACTER*(80)        PLTNAM
      CHARACTER*(WDSIZE)    SHAPE
      CHARACTER*(WDSIZE)    FRAME
 
 
 
C
C     Load plate kernel.
C
 
C       CALL ERRACT ( 'set', 'report' )
 
C      CALL TRCOFF ()
 
C      CALL DOSHAD( -1 )
 
      CALL CHKIN ( 'tpntocc' )
 
      WRITE(*,*) ' '
      WRITE(*,*) 'Enter plate model file name:'
      READ(*,9000) PLTNAM
 9000 FORMAT( A )
      CALL RDPLAT ( PLTNAM )
 
      CALL PLTOT ( NB, NP, NV )
 
      WRITE(*,*) ' '
      WRITE(*,*) 'number of bodies   = ', NB
      WRITE(*,*) 'number of plates   = ', NP
      WRITE(*,*) 'number of vertices = ', NV
      WRITE(*,*) ' '
 
 
      WRITE(*,*) 'enter observer position:'
      READ(*,*) OBS
 
      WRITE(*,*) 'enter boresight direction:'
      READ(*,*) BORDIR
 
      WRITE(*,*) 'enter sun position:'
      READ(*,*) SUN
 
      WRITE(*,*) 'enter plate division parameter:'
      READ(*,*) NDIV
      ITMP = TRIDIV ( NDIV )
      WRITE(*,*) 'tridiv = ',  ITMP
 
      WRITE(*,*) ' '
      WRITE(*,*) 'observer position = ', OBS
      WRITE(*,*) 'boresight direction = ', BORDIR
      WRITE(*,*) 'sun direction      = ', SUN
      WRITE(*,*) ' '
 
 
C
C     Select field of view mode.
C
 
      WRITE(*,*) 'Use an instrument field of view? (y/n)'
      READ(*,9000) FYN
 
 
      IF ( FYN .EQ. 'y' ) THEN
 
C
C     Load instrument kernel.
C
 
         WRITE(*,*) 'Enter name of instrument kernel:'
         READ(*,9000) IKERN
 
         CALL LDPOOL ( IKERN )
 
         WRITE(*,*) 'enter instrument ID:'
         READ(*,*) INSTID
 
C
C     Get the observer ID associated with this instrument
C
 
         OBSID = INSTID / 1000
 
C
C     Determine where the boresight and edge of the field of
C     view are pointing.
C
 
         CALL GETFOV ( INSTID, FVROOM, SHAPE, FRAME, BSIGHT,
     .                 FOVSIZ, EDGE )
 
         CALL UCASE  ( SHAPE,          SHAPE )
 
         IF ( SHAPE .EQ. 'CIRCLE' ) THEN
 
C
C        We generate a field of view for the circular case.
C
 
            ANGLE = TWOPI() / NSIDES
 
            DO I = 1, 15
               CALL VROTV ( EDGE(1,I), BSIGHT, ANGLE, EDGE(1,I+1) )
            END DO
 
            FOVSIZ = 16
 
         ELSE IF ( SHAPE .NE. 'POLYGON' ) THEN
 
            ARTCLE =  ANA ( SHAPE, 'L' )
 
            CALL SETMSG ( 'Fields of view in the shape of # ''#'' '
     .      //            'are not supported by TPNTOCC at the '
     .      //            'present time. ' )
 
            CALL ERRCH  ( '#', ARTCLE )
            CALL ERRCH  ( '#', SHAPE  )
            CALL SIGERR ( 'SPICE(UNSUPPORTEDFOV)'  )
            CALL CHKOUT ( 'TPNTOCC' )
            STOP
 
         END IF
 
C
C     Get the rotation matrix for the field of view from
C     the observer's position and look direction.
C
C     Look vector is towards the center of the body fixed frame.
C
 
C         CALL VMINUS ( OBS, LOOK )
 
         CALL VECMAT ( BSIGHT, BORDIR, XROT )
 
C
C     Rotate all of the FOV components into the bodyfixed frame.
C
 
         CALL MXV ( XROT, BSIGHT, BDSITE )
 
         DO I = 1, FOVSIZ
            CALL MXV ( XROT, EDGE(1,I), FOV(1,I) )
         END DO
 
C
C     Check separation before and after FOV rotation.
C
 
      DO I = 1, FOVSIZ
         WRITE(*,*) 'before: ', I,  VSEP ( BSIGHT, EDGE(1,I) )
         WRITE(*,*) 'after:  ', I,  VSEP ( BDSITE, FOV(1,I) )
      END DO
 
      WRITE(*,*) 'separation of boresight and look vector :',
     .           VSEP( BSIGHT, BDSITE )
 
C
C     Obtain the outward normal vector for each FOV plane.
C
 
         DO I = 1, FOVSIZ-1
            CALL VSUB  ( FOV(1,I+1), FOV(1,I),  SIDE(1,I)   )
            CALL UCRSS ( FOV(1,I),   SIDE(1,I), FOVNRM(1,I) )
         END DO
 
         CALL VSUB  ( FOV(1,1), FOV(1,FOVSIZ),       SIDE(1,FOVSIZ)   )
         CALL UCRSS ( FOV(1,FOVSIZ), SIDE(1,FOVSIZ), FOVNRM(1,FOVSIZ) )
 
         WRITE(*,*) 'FOV outward normals:'
 
         DO I = 1, FOVSIZ
            WRITE(*,*) I, (FOVNRM(J,I), J=1,3)
         END DO
 
      ELSE
C
C     No field of view case.
C
 
         FOVSIZ = 0
 
      END IF
 
C
C     Check each plate for occlusion.
C
 
      IAW = 0
      IFOV = 0
      ILIT = 0
      IVIS = 0
 
      CALL VHAT ( SUN, SUN )
 
      DO I = 1, 8200

         WRITE(*,*) 'tpntocc, in loop.1 = ', I
 
 
         CALL PNTOCC ( I,
     .                 FOVSIZ,  FOV,    FOVNRM,
     .                 OBS,     SUN,
     .                 FOVAMT,  VISAMT, LITAMT,
     .                 AWAY )

        
         WRITE(*,*) 'tpntocc, in loop.2 = ', I
 
 
         IF ( AWAY ) THEN
            IAW = IAW + 1
            WRITE(*,*) 'plate faces away', I
 
         END IF
 
         CALL NTHPUN ( I, NRML )
         WRITE(*,*) 'tpntocc, in loop.3 = ', I
         CALL NTHMSK ( I, PMASK, MAXELE )
         WRITE(*,*) 'tpntocc, in loop.4 = ', I
 
         WRITE(*,*) ' '
         WRITE(*,*) 'plate index  = ', I
         WRITE(*,*) 'plate mask   = ', PMASK
         WRITE(*,*) 'plate normal = ', NRML
         WRITE(*,*) 'maximum horizon elevation = ', MAXELE
         WRITE(*,*) 'fovamt       = ', FOVAMT
         WRITE(*,*) 'visamt       = ', VISAMT
         WRITE(*,*) 'litamt       = ', LITAMT
 
         FOVSAV(I) = FOVAMT
         VISSAV(I) = VISAMT
         LITSAV(I) = LITAMT
 
         IF ( FOVAMT .GT. 0.D0 ) IFOV = IFOV + 1
         IF ( VISAMT .GT. 0.D0 ) IVIS = IVIS + 1
         IF ( LITAMT .GT. 0.D0 ) ILIT = ILIT + 1
 
 
      END DO
 
      WRITE(*,*) ' '
      WRITE(*,*) 'plate occlusion values'
 
      WRITE(*,*) ' '
      WRITE(*,*) ' fovamt = visamt = 0, plate not visible.'
      WRITE(*,*) ' litamt = -1, shadowing not calculated.'
      WRITE(*,*) ' '
 
      WRITE(*,*) 'plate ID     fovamt      visamt      litamt'
      WRITE(*,*) '--------     ------      ------      ------'
 
      CALL TXTOPN ( 'fovplates.pov', FUNIT )
 
      DO I = 1, 8200
 
C         IF ( VISSAV(I) .GT. 0.D0 ) THEN
C         WRITE(*,9010) I,  FOVSAV(I),  VISSAV(I), LITSAV(I)
C 9010    FORMAT(I5, ' ', 3(F12.2, ' ') )
C         END IF
 
         CALL NTHPUN ( I, NRML )
         NRMSUN = VDOT( SUN, NRML )
 
         IF ( LITSAV(I) .NE. -1.D0 ) THEN
 
            IF ( LITSAV(I) .EQ. 0.D0 ) THEN
 
               CALL POVTRI(FUNIT, I, 'RED' )
               WRITE(*,*) 'red', I, NRMSUN
 
            ELSE IF ( LITSAV(I) .LT. 1.D0 ) THEN
 
               CALL POVTRI(FUNIT, I, 'GREEN' )
               WRITE(*,*) 'green', I, NRMSUN
 
            ELSE
 
               CALL POVTRI(FUNIT, I, 'BLUE' )
               WRITE(*,*) 'blue', I, NRMSUN
 
            END IF
 
         END IF
 
      END DO
 
      WRITE(*,*) ' '
      WRITE(*,*) 'summary'
      WRITE(*,*) '-------'
      WRITE(*,*) '# plates facing away = ', IAW
      WRITE(*,*) '# of visible plates  = ', IVIS
      WRITE(*,*) '# plate in FOV       = ', IFOV
      WRITE(*,*) '# of lit plates      = ', ILIT
      WRITE(*,*) ' '
 
 
 
      CALL CHKOUT ( 'tpntocc' )
 
      END
